@@ -1,4 +1,4 @@
 package T12;
 use Object::Simple( base => 'B2', mixins => ['M8', 'M9'], mixins_rename => []);
 
-Object::Simple->end;
+Object::Simple->build_class;
